%% Billi, Soderstrom, and Walsh, Monetary Growth Targeting. May 2021
%  file name: fiscal_psi.m
%% Notes: options
%  1) line 22: datefeed = 'yes' or 'no'
%  2) line 24 budget = 'o', 'r', or 't' for outlays, revenues or total (outlays - revenues 
%  3) line 25 base = 'ave' or 'min' -- uses average output gap or minimum
%  gap (i.e., largest negative gap) as scale factor in calculating psi
%  
%  Note: BilliSoderstrom_fiscl_psi.xlsx contains data through 2021Q1. To
%  calculate average NGDPPOT for all of 2021, the data is read using
%  datafeed or if datefeed = 'no', it is entered on line 158.
%% Notes: data
%  Summary table saved to results_budget for budget = outlays, revenues, or
%  total
%  Debt to GDP series only through 2020Q4 as of 5/29/2021. 
%  3/5/2021: CBO projects debt held by public to GDP ratio to hit 0.98 for 2020.
%            See https://www.cbo.gov/publication/56516#section1
clear all
close all
clc
%% options
datafeed = 'no'; % 'yes' to use Datafeed Toolbox to import data directly from FRED
                 % 'no' to read data from BilliSoderstromWalsh_fiscal_psi.xlsx
budget   = 'o'; % o outlays, r revenue or t total  
base     = 'min';
% set start and end dates to use for recessions periods
r1 = '01-Oct-2007';
r2 = '01-Apr-2009'; 
r3 = '01-Jan-2020'; 
r4 = '01-Jan-2021';
%% Budget data -- American Recovery and Reemployment Act (ARRA)
% Congressional Budget Office, Employment and Economic Output in 2014, Feb. 2015
% https://www.cbo.gov/sites/default/files/114th-congress-2015-2016/reports/49958-ARRA.pdf
% Average annual outlays: 2009-2011: 
arrao = (114 + 235 + 147)/3;
% Average annual revenues: 2009-2011: 
arrar = -(65 + 167 + 2)/3;
% Total budget impact:
arrat = arrao - arrar;
%% Cornavirus Aid, Relief, and Economic Security (CARES) Act
% Preliminary Estimate of the Effects of H.R. 748, the CARES Act, Public Law 116-136, Revised LR748
% https://www.cbo.gov/system/files/2020-04/hr748.pdf, Table 1, p. 30
% Average annual outlays: 2020-2021: 
careso = (938+73+99+134)/2;
% Average annual revenues: 2020-2021
caresr = -(568 + 240)/2;
% Total budget impact:
carest = careso - caresr;
%% American Rescue Plan (ARP)
% https://www.cbo.gov/system/files/2021-03/Estimated_Budgetary_Effects_of_HR_1319_as_passed_0.pdf
% Average annual outlays: 2021: 
arpo = 1088;
% Average annual revenues: 2021
arpr = -76;
% Total budget impact:
arpt = arpo - arpr;

%% date series used               
% Series names in FRED:

series    = ["A091RC1Q027SBEA", "FGEXPND",  "FGRECPT", "FYGFGDQ188S", "GDP", "GDPC1", "GDPPOT", "NGDPPOT", "W014RC1Q027SBEA"];
% Series are renamed as follows:
VarNames  = ["INT_Fed", "EXPEND_Fed", "TAXES_Fed", "DEBT_PCTGDP", "GDP",  "GDPC1", "GDPPOT", "NGDPPOT", "TRANSFERS_Fed"];

startdate = '10/01/2006';
enddate   = '01/01/2021';  
%% fetch data from FRED
if strcmp(datafeed,'yes')
    url       = 'https://fred.stlouisfed.org/';
    c         = fred(url);

    for i = 1:numel(series)
        fprintf('Started loading %s ... ',series{i});
        FredSeries = fetch(c, series{i},startdate,enddate);
        numdates = FredSeries.Data(:,1);
        eval([VarNames{i},'= FredSeries.Data(:,2);'])
        fprintf('....\n')
        if strcmpi(strtrim(FredSeries.Frequency), 'Quarterly')
            fprintf('Quarterly data ... ');
            fprintf('Finished reading %s',series{i});
            fprintf(' Series name is now %s',VarNames(i));
            fprintf('...\n')
        end
    end
    close(c);
    datafeed_yes = [INT_Fed EXPEND_Fed TAXES_Fed DEBT_PCTGDP GDP GDPC1 GDPPOT NGDPPOT TRANSFERS_Fed];
    sfile = 'df_yes';
    save(sfile,'INT_Fed', 'EXPEND_Fed', 'TAXES_Fed', 'DEBT_PCTGDP', 'GDP', 'GDPC1', 'GDPPOT', 'NGDPPOT', 'TRANSFERS_Fed');
 
end
%%
if strcmp(datafeed,'no')
    % Setup the Import Options and import the data
    ser    = ["DATE", series];
    VNames = ["DATE", VarNames];
    opts = spreadsheetImportOptions("NumVariables", numel(series)+1);

    % Specify sheet and range
    opts.Sheet = "Quarterly";
    opts.DataRange = "A189:J246";

    % Specify column names and types
    opts.VariableNames = ser;
    opts.VariableTypes = ["datetime", "double", "double", "double", "double", "double", "double","double", "double", "double"];

    % Specify variable properties
    opts = setvaropts(opts, "DATE", "InputFormat", "");

    % Import the data
    fiscal_data = readtable("BilliSoderstromWalsh_fiscal_psi.xlsx", opts, "UseExcel", false);
    % Clear temporary variables
    clear opts
    % renaming variables
    for i = 1:numel(ser)
        eval([VNames{i},' = fiscal_data.',ser{i}]);
        fprintf('Finished reading %s',ser{i});
        fprintf(' Series name is now %s',VNames(i));
        fprintf('...\n')
    end
numdates = datenum(DATE);
end
%% ARRA and Great Recession 2009Q1 - 2011Q4
eval(['arra',' = arra',budget]);
gap = (GDPC1./GDPPOT)-1;
s1 = find(numdates==datenum('01-Jan-2009'));
s2 = find(numdates==datenum('01-Oct-2011'));
ave_ngdppot0911 = mean(NGDPPOT(s1:s2));
ave_gap0911     = mean(gap(s1:s2));
min_gap0911     = min(gap(s1:s2,1)); % max negative output gap
arra_to_ngdppot = arra/ave_ngdppot0911;
psi_arra1       = -(arra/ave_ngdppot0911)/ave_gap0911;
psi_arra2       = -(arra/ave_ngdppot0911)/min_gap0911;

%% CARES ACT for 2020-2021
% CBO estimated budget impact of CARES Act pased April 2020
eval(['cares',' = cares',budget]);
c1 = find(numdates==datenum('01-Jan-2020'));
c2 = find(numdates==datenum('01-Jan-2021'));
ave_ngdppot00   = mean(NGDPPOT(c1:c2));
ave_gap00       = mean(gap(c1:c2));
min_gap00       = min(gap(c1:c2,1)); % max negative output gap
cares_to_ngdppot= cares/ave_ngdppot00  
psi_cares1      = -(cares/ave_ngdppot00)/ave_gap00;
psi_cares2      = -(cares/ave_ngdppot00)/min_gap00;

%% Reading in NGDPPOT for 2021Q1 - 2021Q4
% if datefeed = 'no', NGDPPOT 2021 is entered in line 154.
startdate = '01/01/2021';
enddate   = '10/01/2021';  
if strcmp(datafeed,'yes')    
    url       = 'https://fred.stlouisfed.org/';
    c         = fred(url);
    FredSeries = fetch(c,'NGDPPOT',startdate,enddate);
    eval(['ngdpp','= FredSeries.Data(:,2);'])
    close(c)
    ave_ngdppot21 = mean(ngdpp);
else
    ngdpp = [22644.98 22983.26 23233.13 23461.34]; % UPDATED 2021-07-17
    ave_ngdppot21 = mean(ngdpp);
end
%%
%c1 = find(numdates==datenum('01-Jan-2020'));
eval(['arp',' = arp',budget]);
%ave_ngdppot21   = mean(NGDPPOT(d2));
ave_gap00       = mean(gap(c1:c2));
min_gap00       = min(gap(c1:c2,1)); % max negative output gap
arp_to_ngdppot  = arp/ave_ngdppot21;
psi_arp1        = -(arp/ave_ngdppot21)/ave_gap00;
psi_arp2        = -(arp/ave_ngdppot21)/min_gap00;
%%
disp(['average gap in 2009Q1-2011Q4 and 2020Q1-2021Q1: ',num2str([ave_gap0911 ave_gap00])])
disp('.......')
disp(['min gap in 2009Q1-2011Q4 and 2020Q1-2021Q1: ',num2str([min_gap0911 min_gap00])])
%%
if strcmp(budget,'o')
    concept = 'outlays';
else
    if strcmp(budget,'r')
        concept = 'revenues';
    else
        concept = 'total';
    end
end
disp('.......')
disp(['budget concept is ',concept])
disp('policy as % of NGDPPOT, psi using average gap, psi using min gap') 
disp('top row - ARRA, middle row CARES, bottom row ARP')
summary = [arra_to_ngdppot psi_arra1 psi_arra2
    cares_to_ngdppot psi_cares1 psi_cares2
    arp_to_ngdppot psi_arp1 psi_arp2]
disp('.......')
disp(['ratio of ave gap psi_cares to psi_arra = ',num2str(psi_cares1/psi_arra1)])
disp('.......')
disp(['ratio of min gap psi_cares to psi_arra = ',num2str(psi_cares2/psi_arra2)])
disp('.......')
disp(['ratio of ave gap psi_arp to psi_arra = ',num2str(psi_arp1/psi_arra1)])
disp('.......')
disp(['ratio of min gap psi_arp to psi_arra = ',num2str(psi_arp2/psi_arra2)])

%% Saving results
psi_avegap = summary(:,2);
psi_mingap = summary(:,3);
savefile = ['results ',concept];
save(savefile,'summary','budget','concept','psi_avegap','psi_mingap')